﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WindowsFormsApp91
{
    internal static class Utils
    {
        #region CaptureControlImageWithChildren
        public static Bitmap CaptureControlImageWithChildren(Control control)
        {
            var bitmap = new Bitmap(control.Width, control.Height);
            CaptureControlImageWithChildren(ref bitmap, control, control.Location);
            return bitmap;
        }

        private static void CaptureControlImageWithChildren(ref Bitmap bitmap, Control control, Point rootControlOrigin)
        {
            var controlsToDraw = GetDescendantControls(control);

            using (Graphics g = Graphics.FromImage(bitmap))
            {
                foreach (Control controlToDraw in controlsToDraw)
                {
                    var controlBounds = GetRelativeControlBounds(controlToDraw, control);
                    controlBounds.Offset(-rootControlOrigin.X, -rootControlOrigin.Y);
                    controlToDraw.DrawToBitmap(bitmap, controlBounds);
                }
            }
        } 
        #endregion // CaptureControlImageWithChildren

        #region GetRelativeControlBounds
        private static Rectangle GetRelativeControlBounds(Control descendantControl, Control rootControl)
        {
            if (descendantControl == rootControl)
                return rootControl.Bounds;

            Rectangle relativeControlBounds = descendantControl.Bounds;
            var parentControl = descendantControl;

            do
            {
                parentControl = parentControl.Parent;
                relativeControlBounds.Offset(parentControl.Left, parentControl.Top);
            }
            while (parentControl != rootControl);

            return relativeControlBounds;
        } 
        #endregion // GetRelativeControlBounds

        #region GetDescendantControls
        private static IEnumerable<Control> GetDescendantControls(Control control)
        {
            List<Control> controls = new List<Control>();
            GetDescendantControls(control, ref controls);
            return controls;
        }

        private static void GetDescendantControls(Control control, ref List<Control> controls)
        {
            controls.Add(control);

            foreach (Control childControl in control.Controls)
            {
                GetDescendantControls(childControl, ref controls);
            }
        } 
        #endregion // GetDescendantControls
    }
}
